/* 
 *  CDDVDSpd Copyright (C) Roy Longbottom January 2004
 *
 *  What the benchmark does
 *
 *  CDDVDspd.exe reads selected files or writes and reads files via a 
 *  selected path. Devices used can include CD, DVD, disks, network 
 *  drives, memory card readers etc. Speed is measured in MBytes Per 
 *  Second, where M = 1048576. These results are saved in (added to) a 
 *  log file SpeedLog.txt.  
 *  
 *  Modules
 * 
 *  Config.cpp - via WhatConfig() identifies CPU type/speed, memory
 *               size, OS version and storage space on selected path,
 *               also running times via getStartTime() and getRunSecs().
 * 
 *  CDDVDSpd.rc - menus and dialog boxes, AboutDlgProc(), ParamData()
 *                
 *  CDDVDSpd.h, CDDVDSpd.ico
 *
 *  ParamData box is displayed when tests are not running and either
 *  displays selected path or test results. The write/read controls are
 *  SELECT_PATH button,  WRRD_BUTTON and AT_MB drop down list for file 
 *  size 1 to 1024 MB. Read only has SELECT_FILE and READ_BUTTON. There
 *  is also an EXIT_BUTTON.
 *
 *  CDDVDSpd.cpp
 *
 *  Particularly with Windows XP, on running a benchmark which displays
 *  on-going results, the window for the latter can be covered by
 *  another window. To overcome this, all tests are run via the WinMain 
 *  message processing loop using a phase variable:
 * 
 *  phase = idleLoop - do nothing
 *  phase = displayLoop - redisplay ParamData() via displayWhatever()
 *  phase = writeLarge - write large file in 1 MB steps via writeafile()
 *  phase = writeSmall - up to 520 files 10 at a time via writeShort()
 *  phase = setUpLread - using setUpRead() for write/read
 *  phase = readLarge - read large file in 1 MB steps via readafile()
 *  phase = saveReadOnly - save results to log file via saveResults()
 *  phase = readSmall - using readShort()
 *  phase = deleteSmall - delete small files using deleteShort()
 *  phase = deleteLarge - using DeleteFile()
 *  phase = saveWrtRd - using saveResults()
 *  
 *  InitApp() sets useMB[] for large files to 1, 2, 4 .. 1024 and
 *  corresponding small files useKB[] to 2, 4, 8 .. 512. One MB of
 *  memory is allocated for data as readData[]
 *
 *  SELECT_FILE and SELECT_PATH buttons call GetFileName() to open
 *  standard Windows Open or Save common dialog boxes
 *
 *  WRRD_BUTTON and READ_BUTTON call setUpWrite() or setUpRead() to
 *  open large files. The appropriate phase is then assigned.
 *
 *  When tests are running the progess and cumulative speed is
 *  displayed in the main window every 1 MB or 10 small files.
 *
 */
 
#define NAME "Test"
#define TITLE " Roy Longbottom's Disk, CD and DVD File Reading and Writing  Benchmark "
#define INITGUID

#include <windows.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <math.h>
#include <malloc.h>
#include <direct.h>
#include <commdlg.h>
#include <time.h>
#include "CDDVDSpd.h"

//  ************** PROTOPTYPES *****************

BOOL FAR PASCAL AboutDlgProc( HWND, UINT, WPARAM, LPARAM);

static BOOL InitApp(HINSTANCE, LPSTR, int);
long FAR PASCAL WindowProc(HWND, UINT, WPARAM, LPARAM);                  
static void CleanUpAndPostQuit(void);
void displayWhatever(HWND);
BOOL FAR PASCAL ParamData( HWND, UINT, WPARAM, LPARAM);

static BOOL GetFileName();
BOOL  readafile();
BOOL  setUpRead();
BOOL  writeafile();
BOOL  setUpWrite();
BOOL  saveResults();
BOOL  writeShort();
BOOL  readShort();
BOOL  deleteShort();
void local_time();


// ************* GLOBAL VARIABLES **********

#define OneMB 1048576
#define OneMW  262144
int    *readData;

WIN32_FIND_DATA W32DiskData;

BOOL bQuit = FALSE;

HWND   mainHWND;
HWND   paramHwnd = NULL;
HWND   progressHwnd = NULL;
HANDLE hFile;
HCURSOR hcur;
HINSTANCE  MyInstance;

DWORD  fileAttrib;

int    useMB[20];
int    useKB[20];
int    countMB = 11;
int    thisMB = 4;  // start at 8 MB
int    shortFiles = 0;
int    shortRead = 0;
int    shortMax = 520;
int    startMax = 520;
int    slowMax = 520;
int    memMax;
int    fileToDelete = 0;
int    displayLoop = 0;
int    idleLoop = -1;
int    phase = -1;
int    writeLarge = 1;
int    setUpLread  = 2;
int    readLarge = 3;
int    saveReadOnly = 4;
int    deleteLarge = 5;
int    writeSmall = 6;
int    readSmall = 7;
int    deleteSmall = 8;
int    saveWrtRd = 9;

char   fileToUse[512] = "No file selected";
char   fileSmall[MAX_PATH] = "No file selected";
char   fileToWrite[32] = "ZTestRW.zzz";
char   fileNumber[20];
char   otherFile[MAX_PATH];
char   currentDir[MAX_PATH];

char   logfile[256] = "SpeedLog.txt";
char   windowText[256];
char   timeday[30];
char   errorMsg[256];
char   version[20] = "Version 1.0";

short  keyPress;

double readSpeed = 0;
double readSeconds = 0;
double writeSpeed = 0;
double writeSeconds = 0;
double readSpeed2 = 0;
double readSeconds2 = 0;
double writeSpeed2 = 0;
double writeSeconds2 = 0;
double deleteSeconds = 0;


BOOL   fileSelected = FALSE;
BOOL   pathSelected = FALSE;
BOOL   justSelected = FALSE;
BOOL   finishedTests = FALSE;
BOOL   readOnly = TRUE;
BOOL   loadIt = TRUE;
BOOL   newMB = FALSE;
BOOL   fileWritten = FALSE;
BOOL   error = FALSE;
BOOL   noSelect = FALSE;
BOOL   escPressed = FALSE;

unsigned long  actualFileSize = 0;
unsigned long  fileMB;
unsigned long  passes;
unsigned long  writeMB;
unsigned long  writeKB;


void displayWhatever(HWND hwnd)
{
    if (paramHwnd)   DestroyWindow(paramHwnd);
    sprintf(windowText, TITLE);
    SetWindowText(mainHWND, windowText);        
    paramHwnd = CreateDialog( MyInstance,"DIALOG_8", hwnd, (DLGPROC)ParamData);
               
} // displayWhatever

/*
 * WinMain - initialization, message loop
 */
int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                     LPSTR cmdline, int cmdshow)
{
    MSG         msg;
    char     buf2[250];    
    char     buf[500];
    
    hPrevInstance = hPrevInstance;

    if (!InitApp(hInstance, cmdline, cmdshow))
         return FALSE;

    while(!bQuit)
    {
        if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
        {
            
           if (msg.message == WM_QUIT)
           {
                CleanUpAndPostQuit();
                break;
           }
   
           if( !GetMessage( &msg, NULL, 0, 0 ) ) return msg.wParam;
           TranslateMessage(&msg);
           DispatchMessage(&msg);
        }
        else if (phase == displayLoop)
        {
            phase = idleLoop;
            SetCursor(LoadCursor(NULL, IDC_ARROW));
            if (justSelected)
            {                      
                 justSelected = FALSE;
                 displayWhatever(mainHWND);
            }
            else if (finishedTests)
            {
                displayWhatever(mainHWND);
                finishedTests = FALSE;
            }
            else if (error)
            {
                displayWhatever(mainHWND);            
                error = FALSE;
            }
            else if (noSelect)
            {
                displayWhatever(mainHWND);            
                noSelect = FALSE;
            }
            else if (newMB)
            {
                displayWhatever(mainHWND);
                newMB = FALSE;
            }
        }
        else if (phase == writeLarge)
        {
            if (writeafile())
            {
                passes = passes + 1;
                if (passes == fileMB)
                {
                    phase = writeSmall;
                    FlushFileBuffers(hFile);
                    CloseHandle(hFile);
                    SetCursor(LoadCursor(NULL, IDC_ARROW));
                    getRunSecs();
                    writeSpeed = (double)(passes) / runSecs;
                    writeSeconds = runSecs;
                    getStartTime();
                }
            }
            else
            {
                phase = deleteLarge;
            }
        }   
        else if (phase == writeSmall)
        {
            SetCurrentDirectory(readDir);
            if (writeShort())
            {
                if (shortFiles == shortMax)
                {
                    getRunSecs();
                    writeSeconds2 = runSecs;
                    writeSpeed2 = (double)(shortFiles) * (double)writeKB / 1024 / runSecs;
                    phase = setUpLread;
                }
            }
            else
            { 
                phase = setUpLread;
                if (escPressed)
                {
                    phase = deleteSmall;
                }
                else if (error)
                {
                    phase = deleteSmall;
                }
                else if (shortFiles < shortMax)
                {
                        sprintf(buf2, "NOTE: could not write %d %d KB files", shortMax, writeKB);
                        sprintf(buf, "Only %d files written", shortFiles);
                        MessageBox(NULL, buf2, buf, MB_ICONWARNING | MB_OK);
                }
           }
           SetCurrentDirectory(currentDir);           
        }
        else if (phase == setUpLread)
        {
            if (setUpRead())
            {
                phase = readLarge;
                passes = 0;
            }
            else
            {
                phase = deleteSmall;
            }                              
        }
        else if (phase == readLarge)
        {
            if (readafile())
            {   
                passes = passes + 1;
                if (passes == fileMB)
                {
                    phase = saveReadOnly;
                    CloseHandle(hFile);
                    SetCursor(LoadCursor(NULL, IDC_ARROW));
                    getRunSecs();
                    readSpeed = (double)(passes) / runSecs;
                    readSeconds = runSecs;
                }
            }
            else
            {
                phase = deleteSmall;
            }
        }
        else if (phase == saveReadOnly)
        {
            if (readOnly)
            {
                finishedTests = TRUE;
                saveResults();
                phase = displayLoop;
            }
            else
            {
                phase = readSmall;
                getStartTime(); 
            }
        }
        else if (phase == readSmall)
        {
            if (shortFiles > 0)
            {
                SetCurrentDirectory(readDir);
                if (readShort())
                {
                    if (shortRead == shortFiles)
                    {
                        getRunSecs();
                        readSeconds2 = runSecs;
                        readSpeed2 = (double)(shortRead) * (double)writeKB / 1024 / runSecs;
                        phase = deleteSmall;
                        getStartTime();                    
                    }
                }
                else
                {
                   phase = deleteSmall;
                   getStartTime();
                }
                SetCurrentDirectory(currentDir);
            }
            else
            {
                phase = deleteLarge;
            }
        }
        else if (phase == deleteSmall)
        {
            if (shortFiles > 0)
            {
                SetCurrentDirectory(readDir);
                deleteShort();
                if (fileToDelete == shortFiles)
                {
                    phase = deleteLarge;
                }
                SetCurrentDirectory(currentDir);
            }
            else
            {
                phase = deleteLarge;
            }
        }        
        else if (phase == deleteLarge)
        {

            if (fileWritten)
            {
                SetCurrentDirectory(readDir);
                DeleteFile(fileToWrite);
                phase = saveWrtRd;
                fileWritten = FALSE;
                SetCurrentDirectory(currentDir);                            
                getRunSecs();
                deleteSeconds = runSecs;
            }
            else 
            {
                phase = displayLoop;
            }
            if (error)
            {
                phase = displayLoop;
            }
        }
        else if (phase == saveWrtRd)
        {
            SetCurrentDirectory(currentDir);
            phase = displayLoop;
            if (!error)
            {
                saveResults();
                finishedTests = TRUE;
            }
            if (shortFiles > 0)
            {                
                if (readSpeed > 20 * readSpeed2 || writeSpeed > 20 * writeSpeed2)
                {
                    sprintf(buf2, "Speed on small files > 20 times slower than large file");
                    sprintf(buf, "Large file write %8.2f MB/sec, read %8.2f MB/sec\n"
                                 "Small file write %8.3f MB/sec, read %8.3f MB/sec\n"
                                 "Is anti-virus checking enabled? - Turn off",
                                 writeSpeed, readSpeed,  writeSpeed2, readSpeed2);
                    MessageBox(NULL, buf, buf2, MB_ICONWARNING | MB_OK);
                }
            }              

        }
        else
        {
           // make sure we go to sleep if we have nothing else to do
           WaitMessage();
        }
    }

    return msg.wParam;
    
} /* WinMain */

/****************************************************************************/
/*                   Initialization and object creation                     */
/****************************************************************************/
/*
 * InitApp
 * Creates window
 */
static BOOL
InitApp(HINSTANCE hInstance, LPSTR cmdline, int cmdshow)
{
    HWND hwnd;
    WNDCLASS wc;
    LPCTSTR  dir = NULL;
    char *delims = {" ,"};
    char buf[1024] = "";
    int  i;

    cmdline = cmdline;
    
    /*
     * set up and registers the window class
     */
    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc = WindowProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = sizeof(DWORD);
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon(hInstance, "GenericIcon");
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName = "GenericMenu";
    wc.lpszClassName = NAME;
    if (!RegisterClass(&wc))
        return FALSE;

    /*
     * Create the window
     */

    hwnd =
        CreateWindow(
            NAME,
            TITLE,
            WS_OVERLAPPEDWINDOW | WS_HSCROLL | WS_VSCROLL,
            0,
            0,
            640,
            360,
            NULL, 
            NULL, 
            hInstance, 
            NULL 
        );

    if (!hwnd) return FALSE;

    mainHWND = hwnd;
    useMB[0] = 1;
    useKB[0] = 2;
    for (i=1; i<countMB; i++)
    {
        useMB[i] = useMB[i-1] * 2;
        useKB[i] = useKB[i-1] * 2;
    }
    strcpy(readDir, " "); 

  
    // Display the window
   
    ShowWindow(hwnd, cmdshow);    
    UpdateWindow(hwnd);

    readData = (int *)VirtualAlloc(NULL, OneMB, MEM_COMMIT, PAGE_READWRITE);
    if (readData == NULL)
    {
        MessageBox(NULL, "Cannot Allocate Memory", "ERROR",
                       MB_ICONWARNING | MB_OK);
        return FALSE;
    }

    keyPress = GetAsyncKeyState(VK_ESCAPE);
    hcur = LoadCursor(NULL, IDC_WAIT);

    displayWhatever(mainHWND);
    GetCurrentDirectory(MAX_PATH, currentDir);
 
    return TRUE;
    
}  // InitApp


/*
 * WindowProc - handle messages for the main application window
 */

long FAR PASCAL WindowProc(HWND hwnd, UINT msg,
                            WPARAM wparam, LPARAM lparam)                  
{     
    switch( msg )
    {
    case WM_ACTIVATEAPP:
        break;
        
    case WM_INITMENUPOPUP:
        break;
        
    case WM_CREATE:
        break;

    case WM_DESTROY:
        CleanUpAndPostQuit();
        break;

    case WM_COMMAND:
     switch( LOWORD( wparam ) )
     {
        case MENU_ABOUT:
            DialogBox( MyInstance,"AboutBox", hwnd, (DLGPROC)AboutDlgProc );
            break;

        case MENU_EXIT:
            CleanUpAndPostQuit();
            break;
                
     }
        case WM_PAINT:
             break;
           
        case WM_KEYDOWN:
            break;

       case WM_ACTIVATE:
            break;
    } 
    return DefWindowProc( hwnd, msg, wparam, lparam );

} /* WindowProc */



/*
 * AboutDlgProc - processes messages for the about dialog.
 */
BOOL FAR PASCAL AboutDlgProc( HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam )
{
    lparam = lparam;

    switch( msg ) {
    case WM_INITDIALOG:
        return( TRUE );

    case WM_COMMAND:
        if( LOWORD( wparam ) == IDOK ) {
            EndDialog( hwnd, TRUE );
            return( TRUE );
        }
        break;
    }
    return( FALSE );

} /* AboutDlgProc */




/*
 * CleanUpAndPostQuit
 * Release all D3DRM objects, post a quit message and set the bQuit flag
 */
void
CleanUpAndPostQuit(void)
{
    if (readData) VirtualFree(readData, 0, MEM_RELEASE);
    bQuit = TRUE;
    return;
}


BOOL FAR PASCAL ParamData( HWND hdlg, UINT msg,
                                WPARAM wparam, LPARAM lparam )
{
    wparam = wparam;
    lparam = lparam;
    char     buf[512];
    char     buf2[50];
    char     buf3[100];
    char     buf4[100];
    char     tab = 9;
    int      index;
    int      k;
    int FAR  tabs[11] = {40, 65, 90, 100, 150, 180, 210, 240, 270, 300, 330};

     switch( msg )
     {
       case WM_INITDIALOG:
        SendDlgItemMessage(hdlg, COMMAND_IN, LB_SETTABSTOPS, (WPARAM) 11, (LPARAM) (int FAR*) tabs); 
        SendDlgItemMessage(hdlg, COMMAND_IN, LB_SETHORIZONTALEXTENT,1000, 0L);
        strcpy(buf, " ");
        if (fileSelected || pathSelected)
        {
             strcpy(buf, readDir);
             if (lstrlen(readDir) > 3) strcat(buf, "\\");
        }
        strcat(buf, fileToUse);
        if (fileSelected || pathSelected)
        {
             if (!loadIt) fileMB = writeMB; 
             sprintf(buf2, "  %d MB", fileMB);
             strcat(buf, buf2);
        }
        SendDlgItemMessage(hdlg, COMMAND_IN, LB_ADDSTRING, 0, (LPARAM) buf);
        if (error)
        {
            SendDlgItemMessage(hdlg, COMMAND_IN, LB_ADDSTRING, 0, (LPARAM) errorMsg);
            sprintf(buf3, "Test terminated and results not saved ");
            SendDlgItemMessage(hdlg, COMMAND_IN, LB_ADDSTRING, 0, (LPARAM) buf3);
        }
        else if (finishedTests)
        {
            if (!readOnly)
            {
                sprintf(buf3, "Writing %c%8.2f seconds,%c Speed %8.2f MB/sec.", tab, writeSeconds, tab, writeSpeed);
                SendDlgItemMessage(hdlg, COMMAND_IN, LB_ADDSTRING, 0, (LPARAM) buf3);
            }
            sprintf(buf3, "Reading %c%8.2f seconds,%c Speed %8.2f MB/sec.", tab, readSeconds, tab, readSpeed);
            SendDlgItemMessage(hdlg, COMMAND_IN, LB_ADDSTRING, 0, (LPARAM) buf3);
            if (!readOnly)
            {
                sprintf(buf3, "Writing and Reading %d %d KB files", shortFiles, writeKB);
                SendDlgItemMessage(hdlg, COMMAND_IN, LB_ADDSTRING, 0, (LPARAM) buf3);
                sprintf(buf3, "Writing %c%8.2f seconds,%c Speed %8.3f MB/sec.", tab, writeSeconds2, tab, writeSpeed2);
                SendDlgItemMessage(hdlg, COMMAND_IN, LB_ADDSTRING, 0, (LPARAM) buf3);
                sprintf(buf3, "Reading %c%8.2f seconds,%c Speed %8.3f MB/sec.", tab, readSeconds2, tab, readSpeed2);
                SendDlgItemMessage(hdlg, COMMAND_IN, LB_ADDSTRING, 0, (LPARAM) buf3);
                sprintf(buf3, "Delete %c%8.2f seconds,%c for %d files + 1", tab, deleteSeconds, tab, shortFiles);
                SendDlgItemMessage(hdlg, COMMAND_IN, LB_ADDSTRING, 0, (LPARAM) buf3);
            }
            sprintf(buf4, "   ");
            SendDlgItemMessage(hdlg, COMMAND_IN, LB_ADDSTRING, 0, (LPARAM) buf4);
            sprintf(buf4, "Saved in log file %s", logfile);
            SendDlgItemMessage(hdlg, COMMAND_IN, LB_ADDSTRING, 0, (LPARAM) buf4);
        }
        else if (fileSelected || pathSelected)
        {
            if (loadIt)
            {
                sprintf(buf2, "Read Only           ");
                SendDlgItemMessage(hdlg, COMMAND_IN, LB_ADDSTRING, 0, (LPARAM) buf2);
            }
            else
            {
                sprintf(buf2, "Write and Read");
                SendDlgItemMessage(hdlg, COMMAND_IN, LB_ADDSTRING, 0, (LPARAM) buf2);
                sprintf(buf2, "Then Write and Read %d x %d KB files", shortMax, writeKB);
                SendDlgItemMessage(hdlg, COMMAND_IN, LB_ADDSTRING, 0, (LPARAM) buf2);
                sprintf(buf2, "Note: the run time is as seen by the program and such as CDRW");
                SendDlgItemMessage(hdlg, COMMAND_IN, LB_ADDSTRING, 0, (LPARAM) buf2);
                sprintf(buf2, "might not finish I/O activity until later - watch the lights");
                SendDlgItemMessage(hdlg, COMMAND_IN, LB_ADDSTRING, 0, (LPARAM) buf2);
            }
        }
        for (k=0; k<countMB; k++)
        {
            sprintf(buf, "%5d",  useMB[k]);
            SendDlgItemMessage(hdlg, AT_MB, CB_ADDSTRING, 0, (LPARAM) buf);
        }
        SendDlgItemMessage(hdlg, AT_MB, CB_SETCURSEL, thisMB-1, 0L);        
        writeMB = useMB[thisMB-1];
        writeKB = useKB[thisMB-1];
        return( TRUE );
                    
       case WM_COMMAND:
 
        if( HIWORD( wparam ) == CBN_SELCHANGE)
        {
            switch (LOWORD (wparam))
            {
                case AT_MB:
                 index = (int)SendDlgItemMessage(hdlg, AT_MB, CB_GETCURSEL, 0, 0);
                 writeMB = useMB[index];
                 writeKB = useKB[index];
                 thisMB = index + 1;
                 newMB = TRUE;
                 phase = displayLoop;
                 break;                
            }
        }
        else if( HIWORD( wparam ) == BN_CLICKED)
        {
            switch (LOWORD (wparam))
            {
                case READ_BUTTON:
                if (!fileSelected)
                {
                    MessageBox(NULL, "No file selected", "Read Selected File",
                                           MB_ICONWARNING | MB_OK);
                    sprintf(fileToUse, "No file selected");
                    pathSelected = FALSE;
                    noSelect = TRUE;
                    phase = displayLoop;                      
                }
                else
                {
                    if (paramHwnd)   DestroyWindow(paramHwnd);
                    sprintf(windowText, "Reading File %s, %d MB", fileToUse, fileMB);
                    SetWindowText(mainHWND, windowText);
                    keyPress = GetAsyncKeyState(VK_ESCAPE);
                    readOnly = TRUE;
                    if (setUpRead())
                    {
                        phase = readLarge;
                        passes = 0;
                    }
                    else
                    {
                        phase = deleteLarge;
                    }                              
                }
                break;                 

                case SELECT_FILE:
                 loadIt = TRUE;
                 strcpy (fileToUse, "Failed to select file");
                 if (GetFileName())
                 {
                      FindFirstFile(otherFile, &W32DiskData);
                      strcpy (fileToUse, W32DiskData.cFileName);
                      actualFileSize = W32DiskData.nFileSizeLow;
                      fileMB = actualFileSize >> 20;
                      if (fileMB == 0)
                      {
                          sprintf(buf2, " File Size  %d bytes", actualFileSize);
                          MessageBox(NULL, buf2, "File MUST be at least 1 MB",
                                                              MB_ICONWARNING | MB_OK);
                          sprintf(fileToUse, "No file selected");                       
                          noSelect = TRUE;
                          phase = displayLoop;
                          fileSelected = FALSE;
                          pathSelected = FALSE;                     
                      }
                      else
                      {
                           fileSelected = TRUE;
                           pathSelected = FALSE;
                           justSelected = TRUE;
                           phase = displayLoop;
                           GetCurrentDirectory(MAX_PATH, readDir);
                      }
                 }
                 else
                 {
                     fileSelected = FALSE;
                     pathSelected = FALSE;                     
                     noSelect = TRUE;
                     phase = displayLoop;
                     MessageBox(NULL, fileToUse, "ERROR", MB_ICONWARNING | MB_OK);
                     sprintf(fileToUse, "No file selected");                       
                 }
                 break;

                 case SELECT_PATH:
                 loadIt = FALSE;
                 strcpy (fileToUse, fileToWrite);
                 if (GetFileName())
                 {
                     pathSelected = TRUE;
                     fileSelected = FALSE;
                     justSelected = TRUE;
                     phase = displayLoop;
                     GetCurrentDirectory(MAX_PATH, readDir);
                 }
                 else
                 {
                     pathSelected = FALSE;
                     strcpy (fileToUse, "Failed to select file");
                     MessageBox(NULL, fileToUse, "ERROR", MB_ICONWARNING | MB_OK);
                     sprintf(fileToUse, "No file selected");                       
                     fileSelected = FALSE;
                     pathSelected = FALSE;                     
                     noSelect = TRUE;
                     phase = displayLoop;                      
                 }                 
                 break;
                 
                case WRRD_BUTTON:
                if (!pathSelected)
                {
                    MessageBox(NULL, "No path selected", "Write/Read to Selected Path",
                                           MB_ICONWARNING | MB_OK);
                    sprintf(fileToUse, "No file selected");                       
                    fileSelected = FALSE;                     
                    noSelect = TRUE;
                    phase = displayLoop;                      
                }
                else
                {
                    if (paramHwnd)   DestroyWindow(paramHwnd);
                    sprintf(windowText, "Writing & Reading Path %s", readDir);
                    SetWindowText(mainHWND, windowText);
                    keyPress = GetAsyncKeyState(VK_ESCAPE);       
                    readOnly = FALSE;
                    if (setUpWrite())
                    {
                        fileWritten = TRUE;
                        phase = writeLarge;
                        passes = 0;
                    }
                    else
                    {
                        phase = displayLoop;
                        error = TRUE;
                    }
                }
                break;

                 case EXIT_BUTTON:
                 PostMessage(mainHWND, WM_COMMAND, MENU_EXIT, 0);
                 break;
            }
        }
        return( TRUE );         
     }
     return( FALSE );
    
} // ParamData

BOOL setUpRead()
{
    HDC  hdc;
    char buf[512];

    readSpeed = 0;
    readSeconds = 0;
    error = FALSE;    
    SetCursorPos(200, 200);

    SetCurrentDirectory(readDir);
    if (readOnly) local_time();
    hdc = GetDC(mainHWND);
    sprintf(buf, " File %s\\%s, %d MB    ", readDir, fileToUse, fileMB);
    TextOut(hdc, 10, 40, buf, lstrlen(buf));
    sprintf(buf, " Opening File For Reading                                                           "
                 "                                              .");
    TextOut(hdc, 10, 80, buf, lstrlen(buf));
    sprintf(buf, "Presss ESC to end reading      ");
    TextOut(hdc, 140, 120, buf, lstrlen(buf));
    UpdateWindow(mainHWND);
    ReleaseDC(mainHWND, hdc);
    getStartTime();
    hFile = CreateFile(fileToUse, GENERIC_READ,
             NULL, NULL, OPEN_EXISTING,
             FILE_ATTRIBUTE_NORMAL | FILE_FLAG_SEQUENTIAL_SCAN
                                | FILE_FLAG_NO_BUFFERING, NULL);
    if (hFile == INVALID_HANDLE_VALUE)
    {
        sprintf(errorMsg, "Failed to open file");
        error = TRUE;
        MessageBox(NULL, errorMsg, "ERROR", MB_ICONWARNING | MB_OK);
        return FALSE;
    }
    return TRUE;
} // setUpRead


BOOL  readafile()
{
    HDC  hdc;
    char buf[512];
    DWORD BytesIn;

    SetCursor(hcur);        
    if (!ReadFile(hFile, readData, OneMB, &BytesIn, NULL))
    {
        sprintf(errorMsg, "Error reading file");
        error = TRUE;
        MessageBox(NULL, errorMsg, "ERROR", MB_ICONWARNING | MB_OK);
        CloseHandle(hFile);
        SetCursor(LoadCursor(NULL, IDC_ARROW));
        return FALSE;
    }          
    getRunSecs();
    readSpeed = (double)(passes + 1) / runSecs;
    hdc = GetDC(mainHWND);
    sprintf(buf, " Read %s\\%s, %d MB    ", readDir, fileToUse, fileMB);
    TextOut(hdc, 10, 40, buf, lstrlen(buf));
    sprintf(buf, "Presss ESC to end reading       ");
    TextOut(hdc, 140, 120, buf, lstrlen(buf));
    sprintf(buf,  " Run time %8.2f seconds for %d MB at %8.2f MB/second, %5.2f Percent Read                     ",
                                 runSecs, passes+1, readSpeed, 100.0 * (double)(passes + 1) / fileMB);
    TextOut(hdc, 10, 80, buf, lstrlen(buf));
    GdiFlush();
    UpdateWindow(mainHWND);
    ReleaseDC(mainHWND, hdc);
    if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
    {
        sprintf(errorMsg, "Escape Pressed");
        error = TRUE;
        finishedTests = FALSE;
        CloseHandle(hFile);
        SetCursor(LoadCursor(NULL, IDC_ARROW));
        return FALSE;
    }
    return TRUE;
    
} // readafile



static BOOL GetFileName()
{
    static char    filterList[] = "File (*.*)" \
                                        "\0" \
                                        "*.*" \
                                        "\0\0";
    OPENFILENAME        of;
    int                 rc;
    
    if (loadIt)
    {
        otherFile[ 0 ] = 0;
    }
    else
    {
        strcpy(otherFile, "Default file name used. Save Path");
    }
    memset( &of, 0, sizeof( OPENFILENAME ) );
    of.lStructSize = sizeof( OPENFILENAME );
    of.hwndOwner = mainHWND;
    of.lpstrFilter = (LPSTR) filterList;
    of.lpstrDefExt = "";
    of.nFilterIndex = 1L;
    of.lpstrFile = otherFile;
    of.nMaxFile = _MAX_PATH;
    of.lpstrTitle = NULL;
    of.Flags = OFN_HIDEREADONLY;

    if (loadIt)
    {
        rc = GetOpenFileName( &of );
    }
    else
    {
        rc = GetSaveFileName( &of );
    }
    return( rc );
    
} // GetFileName

void local_time()
{
    time_t time_of_day;
    time_of_day = time( NULL ); 
    strcpy (timeday, ctime(&time_of_day)); // timeday = date and time
    return;
}

BOOL saveResults()
{
    FILE    *outfile;

    SetCurrentDirectory(currentDir);
    outfile = fopen(logfile,"a+");
    if (outfile == NULL)
    {
        sprintf(errorMsg, "Cannot Open Log File");
        error = TRUE;
        MessageBox(NULL, "Cannot Open Log File", "ERROR", MB_ICONWARNING | MB_OK);
        return FALSE;
    }

    fprintf(outfile, "\n **********************************************************************");
    if (readOnly)
    {
        fprintf(outfile, "\n           Disk, CD and DVD File Reading Test %s", version);
    }
    else
    {
        fprintf(outfile, "\n      Disk, CD and DVD File Writing and Reading Test %s", version);
    }
    fprintf(outfile, "\n                 Copyright (C) Roy Longbottom 2004");
    fprintf(outfile, "\n                 Start at %s", timeday);
  
    if (!readOnly)
    {
        fprintf(outfile, "\n Write %s\\%s, %d MB, %8.2f seconds, %8.2f MB/sec.",
                                    readDir, fileToUse, fileMB, writeSeconds, writeSpeed);
        fprintf(outfile, "\n Read  %s\\%s, %d MB, %8.2f seconds, %8.2f MB/sec.\n",
                                    readDir, fileToUse, fileMB, readSeconds, readSpeed);
        fprintf(outfile, "\n Writing and Reading %d %d KB files in %s\\\n", shortFiles, writeKB, readDir);
        if (shortFiles < shortMax)
        {
            fprintf(outfile, "\n NOTE: could not write %d files\n\n", shortMax);
        }
        else if (memMax < startMax)
        {
            fprintf(outfile, "\n NOTE: Insufficient space for %d short files\n\n", startMax);
        }
        else if (slowMax < startMax)
        {
            fprintf(outfile, "\n NOTE: Excessive time, files reduced from %d to %d\n\n", startMax, slowMax);
        }        
        
        fprintf(outfile, " Writing %8.2f seconds, Speed %8.3f MB/sec.\n", writeSeconds2, writeSpeed2);
        fprintf(outfile, " Reading %8.2f seconds, Speed %8.3f MB/sec.\n", readSeconds2, readSpeed2);
        fprintf(outfile, " Delete  %8.2f seconds, for %d files + 1\n", deleteSeconds, shortFiles);
        if (shortFiles > 0)
        {
            if (readSpeed > 20 * readSpeed2 || writeSpeed > 20 * writeSpeed2)
            {
                fprintf(outfile, "\n Speed on small files > 20 times slower than large file");
                fprintf(outfile, "\n Is anti-virus checking enabled? - Turn off\n");
            }               
        }
        fprintf(outfile, "\n Note: the run time is as seen by the program and such as CDRW\n");
        fprintf(outfile, " might not finish I/O activity until later.\n");
    }
    else
    {
        fprintf(outfile, "\n File %s\\%s, %d MB, %8.2f seconds, %8.2f MB/sec.\n",
                                    readDir, fileToUse, fileMB, readSeconds, readSpeed);
    }
    local_time();

    SetCurrentDirectory(readDir);
    WhatConfig();
    SetCurrentDirectory(currentDir);
     
    fprintf(outfile, "\n                 End at %s", timeday);
    fprintf(outfile, "\n MB in this case is %d bytes. Run time includes opening file.\n", OneMB); 
    fprintf(outfile,"%s\n", configData1);
    fprintf(outfile,"%s\n", configData2);
    fprintf(outfile,"%s\n", configData3);
    fprintf(outfile," Path %s Size %d MB, Free %d MB \n", readDir, (int)totalDiskMB, (int)freeDiskMB);
    

    fflush(outfile);
    fclose(outfile);
    return TRUE; 
} // saveResults

BOOL setUpWrite()
{
    HDC  hdc;
    char buf[512];
    int  i;
    char myMsg[100];
    double thisKB;

    readSpeed = 0;
    readSeconds = 0;
    writeSpeed = 0;
    writeSeconds = 0;
    readSpeed2 = 0;
    readSeconds2 = 0;
    writeSpeed2 = 0;
    writeSeconds2 = 0;
    escPressed = FALSE;           
    shortMax = startMax;
    slowMax  = startMax;
    memMax = startMax;
    shortFiles = 0;
    shortRead = 0;
    fileToDelete = 0;
    error = FALSE;
    fileWritten = FALSE;
    
    SetCursorPos(200, 200);
    
    SetCurrentDirectory(readDir);
    WhatConfig();
    if (freeDiskMB < (float)fileMB + 11.0 * (float)writeKB /1024)
    {
       sprintf(errorMsg, "Insufficient free space");
       MessageBox(NULL, errorMsg, "ERROR", MB_ICONWARNING | MB_OK);
       return FALSE;
    }
    if (freeDiskMB < (float)fileMB + 520.0 * (float)writeKB /1024)
    {
       thisKB = (freeDiskMB - (double)fileMB) * 1024.0;
       shortMax = (int)(thisKB / (float)writeKB / 10.0);
       shortMax = shortMax * 10;
       sprintf(errorMsg, "Insufficient free space for 520 files");
       sprintf(myMsg, " %d short files used", shortMax);
       MessageBox(NULL, myMsg, errorMsg, MB_ICONWARNING | MB_OK);
       slowMax  = shortMax;
       memMax = shortMax;
    }

    hdc = GetDC(mainHWND);
    sprintf(buf, " Write %s\\%s, %d MB     ", readDir, fileToUse, fileMB);
    TextOut(hdc, 10, 40, buf, lstrlen(buf));
    sprintf(buf, "Presss ESC to end writing          ");
    sprintf(buf, " Opening File For Writing                                                           ");
    TextOut(hdc, 10, 80, buf, lstrlen(buf));
    TextOut(hdc, 140, 120, buf, lstrlen(buf));
    sprintf(buf, "In some cases, due to buffering, the progress display may not be updated.");
    TextOut(hdc, 10, 160, buf, lstrlen(buf));
    sprintf(buf, "The light for the drive should indicate that it is being used.");
    TextOut(hdc, 10, 180, buf, lstrlen(buf));
    UpdateWindow(mainHWND);
    ReleaseDC(mainHWND, hdc);

    for (i=0; i<OneMW; i++)
    {
        readData[i] = i;
    }

    local_time();
    getStartTime();
   
    hFile = CreateFile(fileToUse, GENERIC_WRITE,
          NULL, NULL, CREATE_ALWAYS,
          FILE_ATTRIBUTE_NORMAL | FILE_FLAG_NO_BUFFERING, NULL); 
    if (hFile == INVALID_HANDLE_VALUE)
    {
        sprintf(errorMsg, "Failed to open file");
        error = TRUE;
        MessageBox(NULL, errorMsg, "ERROR", MB_ICONWARNING | MB_OK);
        return FALSE;
    }
    return TRUE;
} // setUpWrite


BOOL  writeafile()
{
    HDC  hdc;
    char buf[512];
    DWORD BytesOut;
        
    SetCursor(hcur);
    if (!WriteFile(hFile, readData, OneMB, &BytesOut, NULL))
    {
        sprintf(errorMsg, "Error writing file");
        error = TRUE;
        MessageBox(NULL, errorMsg, "ERROR", MB_ICONWARNING | MB_OK);
        CloseHandle(hFile);
        SetCursor(LoadCursor(NULL, IDC_ARROW));
        return FALSE;
    }
    if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
    {
        sprintf(errorMsg, "Escape Pressed");
        error = TRUE;
        finishedTests = FALSE;
        CloseHandle(hFile);
        SetCursor(LoadCursor(NULL, IDC_ARROW));
        return FALSE;
    }
    getRunSecs();
    hdc = GetDC(mainHWND);
    sprintf(buf, " Write %s\\%s, %d MB     ", readDir, fileToUse, fileMB);
    TextOut(hdc, 10, 40, buf, lstrlen(buf));
    sprintf(buf, "Presss ESC to end writing       ");
    TextOut(hdc, 140, 120, buf, lstrlen(buf));
    sprintf(buf, "In some cases, due to buffering, the progress display may not be updated.");
    TextOut(hdc, 10, 160, buf, lstrlen(buf));
    sprintf(buf, "The light for the drive should indicate that it is being used.");
    TextOut(hdc, 10, 180, buf, lstrlen(buf));
    writeSpeed = (double)(passes + 1) / runSecs;
    sprintf(buf, " Run time %8.2f seconds for %d MB at %8.2f MB/second, %5.2f Percent Written                 ",
                                 runSecs, passes+1, writeSpeed, 100.0 * (double)(passes + 1) / fileMB);
    TextOut(hdc, 10, 80, buf, lstrlen(buf));
    GdiFlush();
    ReleaseDC(mainHWND, hdc);
    UpdateWindow(mainHWND);

    return TRUE;

} // writeafile


BOOL  writeShort()
{
    int i;
    HDC  hdc;
    char buf[512];
    DWORD BytesOut;
    
    for (i=0; i<10; i++)
    {
        sprintf(fileSmall, "ZTest%d.zzz", shortFiles);
        hFile = CreateFile(fileSmall, GENERIC_WRITE,
              NULL, NULL, CREATE_ALWAYS,
              FILE_ATTRIBUTE_NORMAL | FILE_FLAG_NO_BUFFERING, NULL); 
        if (hFile == INVALID_HANDLE_VALUE)
        {
            return FALSE;
        }
        SetCursor(hcur);

        if (!WriteFile(hFile, readData, writeKB * 1024, &BytesOut, NULL))
        {
            sprintf(errorMsg, "Error writing file");
            error = TRUE;
            MessageBox(NULL, errorMsg, "ERROR", MB_ICONWARNING | MB_OK);
            CloseHandle(hFile);
            SetCursor(LoadCursor(NULL, IDC_ARROW));
            return FALSE;
        }
        shortFiles = shortFiles + 1;
        if (shortFiles == shortMax)
        {
            FlushFileBuffers(hFile);
        }
        CloseHandle(hFile);
        if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
        {
            sprintf(errorMsg, "Escape Pressed");
            error = TRUE;
            finishedTests = FALSE;
            escPressed = TRUE;
            SetCursor(LoadCursor(NULL, IDC_ARROW));
            return FALSE;
        }
    }
    getRunSecs();
    writeSeconds2 = runSecs;
    hdc = GetDC(mainHWND);
    sprintf(buf, " Writing %d %d KB files to  %s                              ", shortMax, writeKB, readDir);
    TextOut(hdc, 10, 40, buf, lstrlen(buf));
    sprintf(buf, "Presss ESC to end writing       ");
    TextOut(hdc, 140, 120, buf, lstrlen(buf));
    sprintf(buf, "In some cases, due to buffering, the progress display may not be updated.");
    TextOut(hdc, 10, 160, buf, lstrlen(buf));
    sprintf(buf, "The light for the drive should indicate that it is being used.");
    TextOut(hdc, 10, 180, buf, lstrlen(buf));
    writeSpeed2 = (double)(shortFiles) * (double)writeKB / 1024 / runSecs;
    sprintf(buf, "Run time %8.4f seconds for %6.3f MB at %8.2f MB/second, %3d out of %3d Written                   ",
                                 runSecs, (double)(shortFiles) * (double)writeKB / 1024, writeSpeed2, shortFiles, shortMax);
    TextOut(hdc, 10, 80, buf, lstrlen(buf));
    GdiFlush();
    ReleaseDC(mainHWND, hdc);
    UpdateWindow(mainHWND);
    if (writeSeconds2 > 120)
    {
        shortMax = shortFiles;
        slowMax  = shortFiles;
    }

    return TRUE;

} // writeShort


BOOL readShort()
{
    int i;
    HDC  hdc;
    char buf[512];
    DWORD BytesIn;
    
    for (i=0; i<10; i++)
    {
        sprintf(fileSmall, "ZTest%d.zzz", shortRead);
        hFile = CreateFile(fileSmall, GENERIC_READ,
                 NULL, NULL, OPEN_EXISTING,
                 FILE_ATTRIBUTE_NORMAL | FILE_FLAG_SEQUENTIAL_SCAN
                                    | FILE_FLAG_NO_BUFFERING, NULL);
        if (hFile == INVALID_HANDLE_VALUE)
        {
            sprintf(errorMsg, "Failed to open file");
            error = TRUE;
            MessageBox(NULL, errorMsg, "ERROR", MB_ICONWARNING | MB_OK);
            return FALSE;
        }
        SetCursor(hcur);        
        if (!ReadFile(hFile, readData, writeKB * 1024, &BytesIn, NULL))
        {
            sprintf(errorMsg, "Error reading file");
            error = TRUE;
            MessageBox(NULL, errorMsg, "ERROR", MB_ICONWARNING | MB_OK);
            CloseHandle(hFile);
            SetCursor(LoadCursor(NULL, IDC_ARROW));
            return FALSE;
        }
        CloseHandle(hFile);
        shortRead = shortRead + 1;
        if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
        {
            sprintf(errorMsg, "Escape Pressed");
            error = TRUE;
            escPressed = TRUE;
            finishedTests = FALSE;
            SetCursor(LoadCursor(NULL, IDC_ARROW));
            return FALSE;
        }
        if (shortRead == shortFiles)
        {
            i = 10;
        }
    }
    getRunSecs();
    readSeconds2 = runSecs;
    hdc = GetDC(mainHWND);
    sprintf(buf, " Reading %d %d KB files from  %s                              ", shortFiles, writeKB, readDir);
    TextOut(hdc, 10, 40, buf, lstrlen(buf));
    sprintf(buf, "Presss ESC to end reading         ");
    TextOut(hdc, 140, 120, buf, lstrlen(buf));
    sprintf(buf, "In some cases, due to buffering, the progress display may not be updated.");
    TextOut(hdc, 10, 160, buf, lstrlen(buf));
    sprintf(buf, "The light for the drive should indicate that it is being used.");
    TextOut(hdc, 10, 180, buf, lstrlen(buf));
    readSpeed2 = (double)(shortRead) * (double)writeKB / 1024 / runSecs;
    sprintf(buf, "Run time %8.4f seconds for %6.3f MB at %8.2f MB/second, %3d out of %3d Read                      ",
                                 runSecs, (double)(shortRead) * (double)writeKB / 1024, readSpeed2, shortRead, shortFiles);
    TextOut(hdc, 10, 80, buf, lstrlen(buf));
    GdiFlush();
    ReleaseDC(mainHWND, hdc);
    UpdateWindow(mainHWND);
// MessageBox(NULL, "Here", "ERROR", MB_ICONWARNING | MB_OK);
    return TRUE;
} // readShort

BOOL  deleteShort()
{
    int i;
    HDC  hdc;
    char buf[512];

    SetCursor(hcur);
    for (i=0; i<10; i++)
    {
        sprintf(fileSmall, "ZTest%d.zzz", fileToDelete);
        DeleteFile(fileSmall);
        fileToDelete = fileToDelete + 1;
        if (fileToDelete == shortFiles)
        {
            i = 10;
        }
    }
    getRunSecs();
    deleteSeconds = runSecs;
    hdc = GetDC(mainHWND);
    sprintf(buf, " Deleting %d %d KB files from  %s                              ", shortFiles, writeKB, readDir);
    TextOut(hdc, 10, 40, buf, lstrlen(buf));
    sprintf(buf, "Wait until finished                                    ");
    TextOut(hdc, 140, 120, buf, lstrlen(buf));
    sprintf(buf, "In some cases, due to buffering, the progress display may not be updated.");
    TextOut(hdc, 10, 160, buf, lstrlen(buf));
    sprintf(buf, "The light for the drive should indicate that it is being used.");
    TextOut(hdc, 10, 180, buf, lstrlen(buf));
    sprintf(buf, "Delete file time %8.4f seconds for %3d out of %3d files                                                  ",
                                 runSecs, fileToDelete, shortFiles);
    TextOut(hdc, 10, 80, buf, lstrlen(buf));
    GdiFlush();
    ReleaseDC(mainHWND, hdc);
    UpdateWindow(mainHWND);
    return TRUE;
    
} //deleteShort



